package com.example.amazons3_sdk;

import android.content.Context;

import androidx.annotation.NonNull;
import androidx.work.Data;
import androidx.work.Worker;
import androidx.work.WorkerParameters;

import com.amazonaws.auth.BasicAWSCredentials;
import com.amazonaws.regions.Region;
import com.amazonaws.regions.Regions;
import com.amazonaws.services.s3.AmazonS3Client;
import com.amazonaws.services.s3.model.CannedAccessControlList;
import com.amazonaws.services.s3.model.PutObjectRequest;
import com.amazonaws.services.s3.model.PutObjectResult;

import java.io.File;

import io.flutter.Log;

public class AmazonS3Worker extends Worker {
    AmazonS3Client client;

	public AmazonS3Worker(
			@NonNull Context context,
			@NonNull WorkerParameters params
	) {
		super(context, params);
	}

	@Override
	public Result doWork() {
		try {
			return runAppropriateMethod();
		} catch(Exception e) {
			return errorOccurred();
		}
	}

	private Result errorOccurred() {
		return Result.failure();
	}

	private Result runAppropriateMethod() throws Exception {
		String whatMethod = getInputData().getString("whatMethod");

		if (whatMethod == "uploadFile") {
			return uploadFile();
		} else if (whatMethod == "deleteUploadedFile") {
			return deleteUploadedFile();
		} else if (whatMethod == "uploadedFileExists") {
			return uploadedFileExists();
		} else if (whatMethod == "deleteBucketFolder") {
			return deleteBucketFolder();
		} else {
			throw new Exception();
		}
	}

	private Result deleteBucketFolder() {
		return null;
	}

	private Result uploadedFileExists() {
		return null;
	}

	private Result deleteUploadedFile() {
		return null;
	}

	private Result uploadFile() {
		String bucketName = getInputData().getString("bucketName");
		String key = getInputData().getString("key");
		String filePath = getInputData().getString("filePath");
		File file = new File(filePath);

		getClient().putObject(new PutObjectRequest(bucketName, key, file).withCannedAcl(CannedAccessControlList.PublicRead));


        return Result.success(
            getSuccessfulUploadData(bucketName, key)
        );
	}

    private Data getSuccessfulUploadData(String bucketName, String key) {
        return new Data.Builder()
                    .putString("url", getClient().getResourceUrl(bucketName, key))
                    .putString("key", key)
                    .putBoolean("success", true)
                    .build();
    }

    private Region getRegion() {
		String region = getInputData().getString("region");

		if (region.equals("us-east-1")) {
			Log.i("REGION OF UPLOAD BUCKET", "region given: " + region);
			return Region.getRegion(Regions.US_EAST_1);
		} else if (region.equals("us-east-2")) {
			return Region.getRegion(Regions.US_EAST_2);
		} else {
			throw new Error("Region non implémentée: " + region);
		}
	}

	private AmazonS3Client getClient() {
		if (client != null) {
			return client;
		} else {
            String accessKey = getInputData().getString("accessKey");
            String secretKey = getInputData().getString("secretKey");

            BasicAWSCredentials credentials = new BasicAWSCredentials(
                    accessKey,
                    secretKey
            );

            client = new AmazonS3Client(
                    credentials,
                    getRegion()
            );

            return client;
        }

	}

}
