package com.example.amazons3_sdk;

import android.app.Activity;
import android.content.Context;

import androidx.annotation.NonNull;
import androidx.lifecycle.Lifecycle;
import androidx.lifecycle.LifecycleOwner;
import androidx.lifecycle.Observer;
import androidx.work.Data;
import androidx.work.OneTimeWorkRequest;
import androidx.work.WorkInfo;
import androidx.work.WorkManager;

import com.amazonaws.auth.BasicAWSCredentials;
import com.amazonaws.regions.Region;
import com.amazonaws.regions.Regions;
import com.amazonaws.services.s3.AmazonS3Client;

import java.io.File;
import java.util.HashMap;

import io.flutter.Log;
import io.flutter.plugin.common.MethodChannel;

public class FileUploaderAndDeleter implements LifecycleOwner {
    private String accessKey;
    private String secretKey;
    private String bucketName;
    private String region;

    private Context context;
    private Lifecycle lifecycle;
    //private LifecycleOwner activity;


    /// String String => void
    ///
    /// constructeur, nous permet
    /// d'initialise le client nous permettant un acces read/write
    /// vers notre bucket S3
    FileUploaderAndDeleter(
            String accessKey,
            String secretKey,
            String bucketName,
            String region,
            Context context,
            Lifecycle lifecycle
            //LifecycleOwner activity
    ) {
        this.accessKey = accessKey;
        this.secretKey = secretKey;
        this.bucketName = bucketName;
        this.region = region;
        this.context = context;
        this.lifecycle = lifecycle;
        //this.activity = activity;
    }

    /// File String MethodChannel.Result => void
    ///
    /// uploade le fichier vers le path specifié, dans le Bucket S3
    /// dédié a notre appli
    public void uploadFile(String filePath, String key, MethodChannel.Result result) {

        Log.i("LocalFile", filePath);

        // Create the Data object:
        Data myData = getUploadFileInputData(filePath, key);

        // ...then create and enqueue a
        // OneTimeWorkRequest that uses those arguments
        OneTimeWorkRequest uploadWork = createFileUploadWork(myData);


        listenUploadWorkChanges(context, uploadWork, result);

        startFileUploadWork(context, uploadWork);
    }

    private void listenUploadWorkChanges(Context context, OneTimeWorkRequest uploadWork, final MethodChannel.Result result) {
        WorkManager.getInstance(context).getWorkInfoByIdLiveData(uploadWork.getId())
                .observe(this, new Observer<WorkInfo>() {
            @Override
            public void onChanged(WorkInfo workInfo) {
                if (workInfo != null && workInfo.getState().isFinished()) {
                    String url = workInfo.getOutputData().getString("url");
                    String success = workInfo.getOutputData().getString("success");
                    String key = workInfo.getOutputData().getString("key");

                    if (url != null){
                        HashMap resultValues = new HashMap<>();
                        resultValues.put("url", url);
                        resultValues.put("success", success);
                        resultValues.put("key", key);

                        result.success(resultValues);
                    } else {
                        result.error(null, null, null);
                    }

                }
            }
        });
    }

    private void startFileUploadWork(Context context, OneTimeWorkRequest uploadWork) {
        WorkManager.getInstance(context).enqueue(uploadWork);
    }

    private OneTimeWorkRequest createFileUploadWork(Data myData) {
        return new OneTimeWorkRequest.Builder(AmazonS3Worker.class)
                .setInputData(myData)
                .build();
    }

    private Data getUploadFileInputData(String filePath, String key) {
        return new Data.Builder()
                .putString("accessKey", accessKey)
                .putString("secretKey", secretKey)
                .putString("region", region)
                .putString("whatMethod", "uploadFile")
                .putString("bucketName", bucketName)
                .putString("key", key)
                .putString("filePath", filePath)

                // ... and build the actual Data object:
                .build();
    }













































    /// String MethodChannel.Result => void
    ///
    /// supprime le fichier uploadé a l'emplacement du
    // path specifié, dans le Bucket S3
    /// dédié a notre appli
    public void deleteUploadedFile(String uploadedFilePath, MethodChannel.Result result) {

        Log.i("Uploaded file path", uploadedFilePath);

    }























    /// String String  => Boolean
    ///
    /// le fichier uploadé a l'emplacement du
    /// path specifié, existe t'il
    public Boolean uploadedFileExists(String key, String bucketName) {
        return null;///client.doesObjectExist(bucketName, key);
    }

























    public void deleteBucketFolder(String key, MethodChannel.Result result) {

        Log.i("refkey for folder deletion", key);


    }

























    @NonNull
    @Override
    public Lifecycle getLifecycle() {
        return lifecycle;
    }


}
