package com.example.amazons3_sdk;

import android.app.Activity;
import android.content.Context;

import androidx.annotation.NonNull;
import androidx.lifecycle.Lifecycle;

import java.util.HashMap;

import io.flutter.Log;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.lifecycle.FlutterLifecycleAdapter;

/** amazons3_sdkPlugin */
public class amazons3_sdkPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private MethodChannel channel;

  private Context context;
  private Activity activity;

  /// tous les uploaders en cours d'utilisation
  HashMap<String, FileUploaderAndDeleter> uploaderAndDeleters = new HashMap<>();
  private Lifecycle lifecycle;

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "amazons3_sdk");
    channel.setMethodCallHandler(this);

    context = flutterPluginBinding.getApplicationContext();


  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    if (call.method.equals("getPlatformVersion")) {
      result.success("Android " + android.os.Build.VERSION.RELEASE);
    }
    else if (call.method.equals("uploadFile")) {
      getArgsAndUploadFile(call, result);
    }
    else if (call.method.equals("initUploader")) {
      getArgsAndInitUploader(call, result);
    }
    else if (call.method.equals("deleteUploadedFile")) {
      getArgsAndDeleteUploadedFile(call, result);
    }
    else if (call.method.equals("uploadedFileExists")) {
      getArgsAndUploadedFileExists(call, result);
    }
    else if (call.method.equals("deleteBucketFolder")) {
      getArgsAndDeleteBucketFolder(call, result);
    } else {
      result.notImplemented();
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
  }















  private void getArgsAndDeleteBucketFolder(MethodCall call, MethodChannel.Result result) {
    String key = call.argument("key");
    String uploaderKey = call.argument("uploaderKey");

    deleteBucketFolder(key, uploaderKey, result);
  }

  private void getArgsAndUploadedFileExists(MethodCall call, MethodChannel.Result result) {
    String key = call.argument("key");
    String bucketName = call.argument("bucketName");
    String uploaderKey = call.argument("uploaderKey");

    uploadedFileExists(uploaderKey, key, bucketName, result);
  }

  private void getArgsAndDeleteUploadedFile(MethodCall call, MethodChannel.Result result) {
    String key = call.argument("key");
    String uploaderKey = call.argument("uploaderKey");

    deleteUploadedFile(uploaderKey, key, result);
  }

  private void getArgsAndInitUploader(MethodCall call, MethodChannel.Result result) {
    String accessKey = call.argument("accessKey");
    String secretKey = call.argument("secretKey");
    String bucketName = call.argument("bucketName");
    String region = call.argument("region");
    String uploaderKey = call.argument("uploaderKey");

    initUploader(
            context,
            lifecycle,
            uploaderKey,
            accessKey,
            secretKey,
            bucketName,
            region,
            result
    );
  }

  private void getArgsAndUploadFile(MethodCall call, MethodChannel.Result result) {
    String filePath = call.argument("filePath");
    String key = call.argument("key");
    String uploaderKey = call.argument("uploaderKey");

    uploadFile(filePath, key, uploaderKey, result);
  }









  private void deleteBucketFolder(String key, String uploaderKey, MethodChannel.Result result) {
    uploaderAndDeleters.get(uploaderKey).deleteBucketFolder(key, result);
  }

  private void initUploader(
          Context context,
          Lifecycle lifecycle,
          String uploaderKey,
          String accessKey,
          String secretKey,
          String bucketName,
          String region,
          MethodChannel.Result flutterResult
  ) {
    FileUploaderAndDeleter uploader = new FileUploaderAndDeleter(
            accessKey,
            secretKey,
            bucketName,
            region,
            context,
            lifecycle
    );

    uploaderAndDeleters.put(uploaderKey, uploader);

    flutterResult.success(uploaderAndDeleters.size());
  }



  /// String String MethodChannel.result => void
  ///
  /// verifie qu' un fichier uploadé dans s3 existe toujours
  private void uploadedFileExists(String uploaderKey, String key, String bucketName, MethodChannel.Result flutterResult) {
    try {
      Boolean fileExists = uploaderAndDeleters.get(uploaderKey).uploadedFileExists(key, bucketName);

      flutterResult.success(fileExists);
    } catch(Exception e) {
      flutterResult.error("ERR", e.toString(), null);
    }
  }


  /// String MethodChannel.result => void
  ///
  /// supprime un fichier uploadé dans s3
  private void deleteUploadedFile(String uploaderKey, String uploadedFilePath, MethodChannel.Result flutterResult) {
    try {
      uploaderAndDeleters.get(uploaderKey).deleteUploadedFile(uploadedFilePath, flutterResult);

    } catch(Exception e) {
      flutterResult.error("ERR", e.toString(), null);
    }
  }

  /// String String String String MethodChannel.result => String
  ///
  /// dans un bucket s3,
  /// upload un fichier situé au path filePath,
  /// dans le dossier dirPath, fichier nommé fileName
  /// et retourne l'url permettant de dl/stream le fichier
  private void uploadFile(String filePath, String key, String uploaderKey, MethodChannel.Result flutterResult) {
    try {

      Log.i("LocalFile", filePath);

      uploaderAndDeleters.get(uploaderKey).uploadFile(filePath, key, flutterResult);

    } catch(Exception e) {
      flutterResult.error("ERR", e.toString(), null);
    }
  }


  @Override
  public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
      activity = binding.getActivity();
      lifecycle = FlutterLifecycleAdapter.getActivityLifecycle(binding);
      // Use lifecycle as desired.
  }

  @Override
  public void onDetachedFromActivityForConfigChanges() {

  }

  @Override
  public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {

  }

  @Override
  public void onDetachedFromActivity() {

  }
}
